@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -20,6 +20,8 @@
   ==============================================================================
 */
 
+#include <juce_audio_basics/native/juce_mac_CoreAudioTimeConversions.h>
+
 namespace juce
 {
 
@@ -250,8 +252,7 @@ private:
 };
 
 //==============================================================================
-struct iOSAudioIODevice::Pimpl      : public AudioPlayHead,
-                                      public AsyncUpdater
+struct iOSAudioIODevice::Pimpl      : public AsyncUpdater
 {
     Pimpl (iOSAudioIODeviceType* ioDeviceType, iOSAudioIODevice& ioDevice)
         : deviceType (ioDeviceType),
@@ -292,10 +293,8 @@ struct iOSAudioIODevice::Pimpl      : public AudioPlayHead,
             options |= (AVAudioSessionCategoryOptionDefaultToSpeaker
                       | AVAudioSessionCategoryOptionAllowBluetooth);
 
-           #if defined (__IPHONE_10_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_10_0
             if (@available (iOS 10.0, *))
                 options |= AVAudioSessionCategoryOptionAllowBluetoothA2DP;
-           #endif
         }
 
         JUCE_NSERROR_CHECK ([[AVAudioSession sharedInstance] setCategory: category
@@ -564,145 +563,157 @@ struct iOSAudioIODevice::Pimpl      : public AudioPlayHead,
     }
 
     //==============================================================================
-    bool canControlTransport() override                    { return interAppAudioConnected; }
-
-    void transportPlay (bool shouldSartPlaying) override
+    class PlayHead : public AudioPlayHead
     {
-        if (! canControlTransport())
-            return;
+    public:
+        explicit PlayHead (Pimpl& implIn) : impl (implIn) {}
 
-        HostCallbackInfo callbackInfo;
-        fillHostCallbackInfo (callbackInfo);
+        bool canControlTransport() override                    { return canControlTransportImpl(); }
 
-        Boolean hostIsPlaying = NO;
-        OSStatus err = callbackInfo.transportStateProc2 (callbackInfo.hostUserData,
-                                                         &hostIsPlaying,
-                                                         nullptr,
-                                                         nullptr,
-                                                         nullptr,
-                                                         nullptr,
-                                                         nullptr,
-                                                         nullptr);
+        void transportPlay (bool shouldSartPlaying) override
+        {
+            if (! canControlTransport())
+                return;
 
-        ignoreUnused (err);
-        jassert (err == noErr);
+            HostCallbackInfo callbackInfo;
+            impl.fillHostCallbackInfo (callbackInfo);
 
-        if (hostIsPlaying != shouldSartPlaying)
-            handleAudioTransportEvent (kAudioUnitRemoteControlEvent_TogglePlayPause);
-    }
+            Boolean hostIsPlaying = NO;
+            OSStatus err = callbackInfo.transportStateProc2 (callbackInfo.hostUserData,
+                                                             &hostIsPlaying,
+                                                             nullptr,
+                                                             nullptr,
+                                                             nullptr,
+                                                             nullptr,
+                                                             nullptr,
+                                                             nullptr);
 
-    void transportRecord (bool shouldStartRecording) override
-    {
-        if (! canControlTransport())
-            return;
+            ignoreUnused (err);
+            jassert (err == noErr);
 
-        HostCallbackInfo callbackInfo;
-        fillHostCallbackInfo (callbackInfo);
+            if (hostIsPlaying != shouldSartPlaying)
+                impl.handleAudioTransportEvent (kAudioUnitRemoteControlEvent_TogglePlayPause);
+        }
 
-        Boolean hostIsRecording = NO;
-        OSStatus err = callbackInfo.transportStateProc2 (callbackInfo.hostUserData,
-                                                         nullptr,
-                                                         &hostIsRecording,
-                                                         nullptr,
-                                                         nullptr,
-                                                         nullptr,
-                                                         nullptr,
-                                                         nullptr);
-        ignoreUnused (err);
-        jassert (err == noErr);
+        void transportRecord (bool shouldStartRecording) override
+        {
+            if (! canControlTransport())
+                return;
 
-        if (hostIsRecording != shouldStartRecording)
-            handleAudioTransportEvent (kAudioUnitRemoteControlEvent_ToggleRecord);
-    }
+            HostCallbackInfo callbackInfo;
+            impl.fillHostCallbackInfo (callbackInfo);
 
-    void transportRewind() override
-    {
-        if (canControlTransport())
-            handleAudioTransportEvent (kAudioUnitRemoteControlEvent_Rewind);
-    }
+            Boolean hostIsRecording = NO;
+            OSStatus err = callbackInfo.transportStateProc2 (callbackInfo.hostUserData,
+                                                             nullptr,
+                                                             &hostIsRecording,
+                                                             nullptr,
+                                                             nullptr,
+                                                             nullptr,
+                                                             nullptr,
+                                                             nullptr);
+            ignoreUnused (err);
+            jassert (err == noErr);
 
-    bool getCurrentPosition (CurrentPositionInfo& result) override
-    {
-        if (! canControlTransport())
-            return false;
+            if (hostIsRecording != shouldStartRecording)
+                impl.handleAudioTransportEvent (kAudioUnitRemoteControlEvent_ToggleRecord);
+        }
 
-        zerostruct (result);
+        void transportRewind() override
+        {
+            if (canControlTransport())
+                impl.handleAudioTransportEvent (kAudioUnitRemoteControlEvent_Rewind);
+        }
 
-        HostCallbackInfo callbackInfo;
-        fillHostCallbackInfo (callbackInfo);
+        Optional<PositionInfo> getPosition() const override
+        {
+            if (! canControlTransportImpl())
+                return {};
 
-        if (callbackInfo.hostUserData == nullptr)
-            return false;
+            HostCallbackInfo callbackInfo;
+            impl.fillHostCallbackInfo (callbackInfo);
 
-        Boolean hostIsPlaying               = NO;
-        Boolean hostIsRecording             = NO;
-        Float64 hostCurrentSampleInTimeLine = 0;
-        Boolean hostIsCycling               = NO;
-        Float64 hostCycleStartBeat          = 0;
-        Float64 hostCycleEndBeat            = 0;
-        OSStatus err = callbackInfo.transportStateProc2 (callbackInfo.hostUserData,
-                                                         &hostIsPlaying,
-                                                         &hostIsRecording,
-                                                         nullptr,
-                                                         &hostCurrentSampleInTimeLine,
-                                                         &hostIsCycling,
-                                                         &hostCycleStartBeat,
-                                                         &hostCycleEndBeat);
-        if (err == kAUGraphErr_CannotDoInCurrentContext)
-            return false;
+            if (callbackInfo.hostUserData == nullptr)
+                return {};
 
-        jassert (err == noErr);
+            Boolean hostIsPlaying               = NO;
+            Boolean hostIsRecording             = NO;
+            Float64 hostCurrentSampleInTimeLine = 0;
+            Boolean hostIsCycling               = NO;
+            Float64 hostCycleStartBeat          = 0;
+            Float64 hostCycleEndBeat            = 0;
+            OSStatus err = callbackInfo.transportStateProc2 (callbackInfo.hostUserData,
+                                                             &hostIsPlaying,
+                                                             &hostIsRecording,
+                                                             nullptr,
+                                                             &hostCurrentSampleInTimeLine,
+                                                             &hostIsCycling,
+                                                             &hostCycleStartBeat,
+                                                             &hostCycleEndBeat);
+            if (err == kAUGraphErr_CannotDoInCurrentContext)
+                return {};
 
-        result.timeInSamples = (int64) hostCurrentSampleInTimeLine;
-        result.isPlaying     = hostIsPlaying;
-        result.isRecording   = hostIsRecording;
-        result.isLooping     = hostIsCycling;
-        result.ppqLoopStart  = hostCycleStartBeat;
-        result.ppqLoopEnd    = hostCycleEndBeat;
+            jassert (err == noErr);
 
-        result.timeInSeconds = result.timeInSamples / sampleRate;
+            PositionInfo result;
 
-        Float64 hostBeat = 0;
-        Float64 hostTempo = 0;
-        err = callbackInfo.beatAndTempoProc (callbackInfo.hostUserData,
-                                             &hostBeat,
-                                             &hostTempo);
-        jassert (err == noErr);
+            result.setTimeInSamples ((int64) hostCurrentSampleInTimeLine);
+            result.setIsPlaying     (hostIsPlaying);
+            result.setIsRecording   (hostIsRecording);
+            result.setIsLooping     (hostIsCycling);
+            result.setLoopPoints    (LoopPoints { hostCycleStartBeat, hostCycleEndBeat });
+            result.setTimeInSeconds (*result.getTimeInSamples() / impl.sampleRate);
 
-        result.ppqPosition = hostBeat;
-        result.bpm         = hostTempo;
+            Float64 hostBeat = 0;
+            Float64 hostTempo = 0;
+            err = callbackInfo.beatAndTempoProc (callbackInfo.hostUserData,
+                                                 &hostBeat,
+                                                 &hostTempo);
+            jassert (err == noErr);
 
-        Float32 hostTimeSigNumerator = 0;
-        UInt32 hostTimeSigDenominator = 0;
-        Float64 hostCurrentMeasureDownBeat = 0;
-        err = callbackInfo.musicalTimeLocationProc (callbackInfo.hostUserData,
-                                                    nullptr,
-                                                    &hostTimeSigNumerator,
-                                                    &hostTimeSigDenominator,
-                                                    &hostCurrentMeasureDownBeat);
-        jassert (err == noErr);
+            result.setPpqPosition (hostBeat);
+            result.setBpm         (hostTempo);
 
-        result.ppqPositionOfLastBarStart = hostCurrentMeasureDownBeat;
-        result.timeSigNumerator          = (int) hostTimeSigNumerator;
-        result.timeSigDenominator        = (int) hostTimeSigDenominator;
+            Float32 hostTimeSigNumerator = 0;
+            UInt32 hostTimeSigDenominator = 0;
+            Float64 hostCurrentMeasureDownBeat = 0;
+            err = callbackInfo.musicalTimeLocationProc (callbackInfo.hostUserData,
+                                                        nullptr,
+                                                        &hostTimeSigNumerator,
+                                                        &hostTimeSigDenominator,
+                                                        &hostCurrentMeasureDownBeat);
+            jassert (err == noErr);
 
-        result.frameRate = AudioPlayHead::fpsUnknown;
+            result.setPpqPositionOfLastBarStart (hostCurrentMeasureDownBeat);
+            result.setTimeSignature (TimeSignature { (int) hostTimeSigNumerator, (int) hostTimeSigDenominator });
 
-        return true;
-    }
+            result.setFrameRate (AudioPlayHead::fpsUnknown);
+
+            return result;
+        }
+
+    private:
+        bool canControlTransportImpl() const { return impl.interAppAudioConnected; }
+
+        Pimpl& impl;
+    };
 
     //==============================================================================
    #if JUCE_MODULE_AVAILABLE_juce_graphics
     JUCE_BEGIN_IGNORE_WARNINGS_GCC_LIKE ("-Wdeprecated-declarations")
     Image getIcon (int size)
     {
-        if (interAppAudioConnected)
+        if (@available (macCatalyst 14.0, *))
         {
-            UIImage* hostUIImage = AudioOutputUnitGetHostIcon (audioUnit, size);
-            if (hostUIImage != nullptr)
-                return juce_createImageFromUIImage (hostUIImage);
+            if (interAppAudioConnected)
+            {
+                UIImage* hostUIImage = AudioOutputUnitGetHostIcon (audioUnit, size);
+                if (hostUIImage != nullptr)
+                    return juce_createImageFromUIImage (hostUIImage);
+            }
         }
-        return Image();
+
+        return {};
     }
     JUCE_END_IGNORE_WARNINGS_GCC_LIKE
    #endif
@@ -722,7 +733,6 @@ struct iOSAudioIODevice::Pimpl      : public AudioPlayHead,
                                             &dataSize);
         if (err == noErr)
         {
-           #if defined (__IPHONE_10_0) && __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_10_0
             if (@available (iOS 10.0, *))
             {
                 [[UIApplication sharedApplication] openURL: (NSURL*) hostUrl
@@ -731,7 +741,6 @@ struct iOSAudioIODevice::Pimpl      : public AudioPlayHead,
 
                 return;
             }
-           #endif
 
             JUCE_BEGIN_IGNORE_WARNINGS_GCC_LIKE ("-Wdeprecated-declarations")
             [[UIApplication sharedApplication] openURL: (NSURL*) hostUrl];
@@ -900,9 +909,14 @@ struct iOSAudioIODevice::Pimpl      : public AudioPlayHead,
                     zeromem (inputData[c], channelDataSize);
             }
 
-            callback->audioDeviceIOCallback ((const float**) inputData,  channelData.inputs ->numActiveChannels,
-                                                             outputData, channelData.outputs->numActiveChannels,
-                                             (int) numFrames);
+            const auto nanos = time != nullptr ? timeConversions.hostTimeToNanos (time->mHostTime) : 0;
+
+            callback->audioDeviceIOCallbackWithContext ((const float**) inputData,
+                                                        channelData.inputs ->numActiveChannels,
+                                                        outputData,
+                                                        channelData.outputs->numActiveChannels,
+                                                        (int) numFrames,
+                                                        { (time != nullptr && (time->mFlags & kAudioTimeStampHostTimeValid) != 0) ? &nanos : nullptr });
 
             for (int c = 0; c < channelData.outputs->numActiveChannels; ++c)
             {
@@ -1329,6 +1343,8 @@ struct iOSAudioIODevice::Pimpl      : public AudioPlayHead,
         AudioBuffer<float> audioData { 0, 0 };
     };
 
+    CoreAudioTimeConversions timeConversions;
+
     IOChannelData channelData;
 
     BigInteger requestedInputChannels, requestedOutputChannels;
@@ -1370,6 +1386,7 @@ struct iOSAudioIODevice::Pimpl      : public AudioPlayHead,
     Float64 lastSampleTime;
     unsigned int lastNumFrames;
     int xrun;
+    PlayHead playhead { *this };
 
     JUCE_DECLARE_NON_COPYABLE (Pimpl)
 };
@@ -1420,7 +1437,7 @@ int iOSAudioIODevice::getOutputLatencyInSamples()                   { return rou
 int iOSAudioIODevice::getXRunCount() const noexcept                 { return pimpl->xrun; }
 
 void iOSAudioIODevice::setMidiMessageCollector (MidiMessageCollector* collector) { pimpl->messageCollector = collector; }
-AudioPlayHead* iOSAudioIODevice::getAudioPlayHead() const           { return pimpl.get(); }
+AudioPlayHead* iOSAudioIODevice::getAudioPlayHead() const           { return &pimpl->playhead; }
 
 bool iOSAudioIODevice::isInterAppAudioConnected() const             { return pimpl->interAppAudioConnected; }
 #if JUCE_MODULE_AVAILABLE_juce_graphics
