@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -33,9 +33,10 @@ namespace dsp
 template <typename Type>
 struct SIMDRegister<Type>::ElementAccess
 {
-    operator Type() const                                { return simd.get (idx); }
-    ElementAccess& operator= (Type scalar) noexcept      { simd.set (idx, scalar); return *this; }
-    ElementAccess& operator= (ElementAccess& o) noexcept { return operator= ((Type) o); }
+    ElementAccess (const ElementAccess&) = default;
+    operator Type() const                                       { return simd.get (idx); }
+    ElementAccess& operator= (Type scalar) noexcept             { simd.set (idx, scalar); return *this; }
+    ElementAccess& operator= (const ElementAccess& o) noexcept  { return operator= ((Type) o); }
 
 private:
     friend struct SIMDRegister;
